Package: HGC
Type: Package
Title: A fast hierarchical graph-based clustering method
Version: 1.20.0
Authors@R: c(person("Zou", "Ziheng",
                email = "zouzh14@gmail.com",
                role=c("aut")),
             person("Hua", "Kui",
                role=c("aut")),
             person("XGlab",
                email = "xglab@mail.tsinghua.edu.cn",
                role=c("cre", "cph")))
Description: HGC (short for Hierarchical Graph-based Clustering) is an R package for 
        conducting hierarchical clustering on large-scale single-cell RNA-seq 
        (scRNA-seq) data. The key idea is to construct a dendrogram of cells on 
        their shared nearest neighbor (SNN) graph. HGC provides functions for 
        building graphs and for conducting hierarchical clustering on the graph. The
        users with old R version could visit 
        https://github.com/XuegongLab/HGC/tree/HGC4oldRVersion to get HGC package
        built for R 3.6.
License: GPL-3
Encoding: UTF-8
SystemRequirements: C++11
Depends: R (>= 4.1.0)
Imports: Rcpp (>= 1.0.0), RcppEigen(>= 0.3.2.0), Matrix, RANN, ape,
        dendextend, ggplot2, mclust, patchwork, dplyr, grDevices,
        methods, stats
LinkingTo: Rcpp, RcppEigen
Suggests: BiocStyle, rmarkdown, knitr, testthat (>= 3.0.0)
VignetteBuilder: knitr
biocViews: SingleCell, Software, Clustering, RNASeq, GraphAndNetwork,
        DNASeq
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-04-28 22:34:47 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/HGC
git_branch: RELEASE_3_23
git_last_commit: 4afcd45
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
Author: Zou Ziheng [aut],
  Hua Kui [aut],
  XGlab [cre, cph]
Maintainer: XGlab <xglab@mail.tsinghua.edu.cn>

useDynLib(HGC, .registration = TRUE)

export(HGC.dendrogram)
export(HGC.parameter)
export(KNN.Construction)
export(RNN.Construction)
export(CKNN.Construction)
export(SNN.Construction)
export(MST.Construction)
export(PMST.Construction)
export(HGC.PlotDendrogram)
export(HGC.PlotARIs)
export(HGC.PlotParameter)
export(FindClusteringTree)
S3method(HGC.dendrogram, default)
S3method(HGC.dendrogram, dgCMatrix)
S3method(HGC.dendrogram, graph)
S3method(HGC.dendrogram, matrix)
S3method(HGC.dendrogram, igraph)

import(Matrix)
import(Rcpp)
import(RcppEigen)
import(ggplot2)
import(dendextend)
import(dplyr)
importFrom(stats, order.dendrogram)
importFrom(RANN, nn2)
importFrom(ape, mst)
importFrom(grDevices, hcl)
importFrom(mclust, adjustedRandIndex)
importFrom(patchwork, wrap_plots)
importFrom(methods, as)

# HGC: fast hierarchical clustering for large-scale single-cell data
## Introduction

`HGC` (short for Hierarchical Graph-based Clustering) is an R package for 
conducting hierarchical clustering on large-scale single-cell RNA-seq 
(scRNA-seq) data. The key idea is to construct a dendrogram of cells on 
their shared nearest neighbor (SNN) graph. `HGC` provides functions for 
building cell graphs and for conducting hierarchical clustering on the graph. 
Experiments on benchmark datasets showed that `HGC` can reveal the 
hierarchical structure underlying the data, achieve state-of-the-art 
clustering accuracy and has better scalability to large single-cell data. 
For more information, please refer to the preprint of `HGC` on 
[bioRxiv](https://doi.org/10.1101/2021.02.07.430106).

## Installation

`HGC` could be installed from Github.

```{r Github install, eval = FALSE}
if(!require(devtools))
    install.packages("devtools")
devtools::install_github("XuegongLab/HGC")
```

## Quick Start

### Input data
`HGC` takes a matrix as input where row represents cells and column 
represents features. Preprocessing steps like normalization and dimension 
reduction are necessary so that the constructed graph can capture the 
manifold underlying the single-cell data. We recommend users to follow 
the standard preprocessing steps in 
[`Seurat`](https://satijalab.org/seurat/articles/get_started.html). 
As a demo input, we stored the top 25 principal components of the 
Pollen dataset ([Pollen et al.](https://www.nature.com/articles/nbt.2967)) 
in `HGC`. The dataset contains 301 cells with two known labels: labels at 
the tissue level and the cell line level.

```{r, message=FALSE, warning=FALSE}
library(HGC)

data(Pollen)
Pollen.PCs <- Pollen[["PCs"]]
Pollen.Label.Tissue <- Pollen[["Tissue"]]
Pollen.Label.CellLine <- Pollen[["CellLine"]]

dim(Pollen.PCs)
table(Pollen.Label.Tissue)
table(Pollen.Label.CellLine)
```

### Run HGC

There are two major steps for conducting the hierarchical clustering 
with `HGC`: the graph construction step and the dendrogram construction 
step. `HGC` provides functions for
building a group of graphs, including the k-nearest neighbor graph (KNN), 
the shared nearest neighbor graph (SNN), the continuous k-nearest neighbor 
graph (CKNN), etc. These graphs are saved as `dgCMatrix` supported by 
R package `Matrix`. Then `HGC` can directly build a hierarchical tree 
on the graph. A self-built graph or graphs from other pipelines stored 
as `dgCMatrix` are also supported.

```{r}
Pollen.SNN <- SNN.Construction(mat = Pollen.PCs, k = 25, threshold = 0.15)
Pollen.ClusteringTree <- HGC.dendrogram(G = Pollen.SNN)
```

The user could also give `HGC.dendrogram` an adjacency matrix directly, for instance, to read a matrix from `igraph` object and use it to run `HGC`.

```{r}
require(igraph)
g <- sample_gnp(10, 2/10)
G <- as_adjacency_matrix(g, sparse = TRUE)
G.ClusteringTree <- HGC.dendrogram(G = G)
```

The output of `HGC` is a standard tree following the data structure `hclust()` 
in R package `stats`. The tree can be cut into specific number of clusters 
with the function `cutree`.  

```{r}
cluster.k5 <- cutree(Pollen.ClusteringTree, k = 5)
```

### Visualization

With various published methods in R, results of `HGC` can be visualized easily. 
Here we use the R package `dendextend` as an example to visualize the results 
on the Pollen dataset. The tree has been cut into five clusters. And for a 
better visualization, the height of the tree has been log-transformed.

```{r, fig.height = 4.5}
Pollen.ClusteringTree$height = log(Pollen.ClusteringTree$height + 1)
Pollen.ClusteringTree$height = log(Pollen.ClusteringTree$height + 1)

HGC.PlotDendrogram(tree = Pollen.ClusteringTree,
                    k = 5, plot.label = FALSE)
```
We can also add a colour bar of the known label under the dendrogram as a 
comparison of the achieved clustering results.

```{r, fig.height = 4.5}
Pollen.labels <- data.frame(Tissue = Pollen.Label.Tissue,
                            CellLine = Pollen.Label.CellLine)
HGC.PlotDendrogram(tree = Pollen.ClusteringTree,
                    k = 5, plot.label = TRUE, 
                    labels = Pollen.labels)
```

### Evaluation of the clustering results

For datasets with known labels, the clustering results of `HGC` can be 
evaluated by comparing the consistence between the known labels and the 
achieved clusters. Adjusted Rand Index (ARI) is a wildly used statistics 
for this purpose. Here we calculate the ARIs of the clustering results at 
different levels of the dendrogram with the two known labels. 

```{r}
ARI.mat <- HGC.PlotARIs(tree = Pollen.ClusteringTree,
                        labels = Pollen.labels)
```

## Time complexity analysis of HGC

Our work shows that the dendrogram construction in `HGC` has a linear time 
complexity. For advanced users, `HGC` provides functions to conduct time 
complexity analysis on their own data. The construction of the dendrogram 
is a recursive procedure of two steps: 1. find the nearest neighbour pair, 
2. merge the node pair and update the graph. For different data structures of 
graph, there's a trade-off between the time consumptions of the two steps. 
Generally speaking, storing more information about the graph makes it faster 
to find the nearest neighbour pair (step 1) but slower to update the graph 
(step 2). We have experimented several datasets and chosen the best data 
structure for the overall efficiency. 

The key parameters related to the time consumptions of the two steps are the 
length of the nearest neighbor chains and the number of nodes needed to be 
updated in each iteration, respectively (for more details, please refer to 
our [preprint](https://doi.org/10.1101/2021.02.07.430106)).`HGC` provides 
functions to record and visualize these parameters.

```{r}
Pollen.ParameterRecord <- HGC.parameter(G = Pollen.SNN)

HGC.PlotParameter(Pollen.ParameterRecord, parameter = "CL")
HGC.PlotParameter(Pollen.ParameterRecord, parameter = "ANN")
```

# running HGC in existing Seurat object, and save the tree in the object
FindClusteringTree <- function(object, graph.type = "SNN"){
    require(Seurat)
    require(Matrix)

    SeuratDefaultAssay <- Seurat::DefaultAssay(object)

    if(graph.type == "SNN"){
        SNN_name = paste(SeuratDefaultAssay, "_snn", sep = "")
        G = Seurat::as.sparse(object@graphs[[SNN_name]])
        ClusteringTree = HGC.dendrogram(G)
        object@graphs[["ClusteringTree"]] = ClusteringTree
        return(object)
    }else if(graph.type == "KNN"){
        KNN_name = paste(SeuratDefaultAssay, "_nn", sep = "")
        G = Seurat::as.sparse(object@graphs[[KNN_name]])
        G = G + t(G)
        ClusteringTree = HGC.dendrogram(G)
        object@graphs[["ClusteringTree"]] = ClusteringTree
        return(object)
    }else{
        stop("The kind of graph is not used by Seurat.")
    }
}

# running HGC in existing igraph object, and output the clustering tree
# we will try to save the tree in the igraph object in later version
cluster_HGC <- function(graph){
    require(igraph)
    G <- igraph::as_adjacency_matrix(graph, sparse = TRUE)
    G.ClusteringTree <- HGC.dendrogram(G = G)
    return(G.ClusteringTree)
}

## ----knitr-options, echo=FALSE, message=FALSE, warning=FALSE------------------
library(knitr)
opts_chunk$set(fig.align = 'center', 
                fig.width = 4.5, fig.height = 3, dev = 'png')

## ----Bioconductor install, eval = FALSE---------------------------------------
# if (!requireNamespace("BiocManager"))
#     install.packages("BiocManager")
# BiocManager::install("HGC")

## ----Github install, eval = FALSE---------------------------------------------
# if(!require(devtools))
#     install.packages("devtools")
# devtools::install_github("XuegongLab/HGC")

## ----message=FALSE, warning=FALSE---------------------------------------------
library(HGC)

data(Pollen)
Pollen.PCs <- Pollen[["PCs"]]
Pollen.Label.Tissue <- Pollen[["Tissue"]]
Pollen.Label.CellLine <- Pollen[["CellLine"]]

dim(Pollen.PCs)
table(Pollen.Label.Tissue)
table(Pollen.Label.CellLine)

## -----------------------------------------------------------------------------
Pollen.SNN <- SNN.Construction(mat = Pollen.PCs, k = 25, threshold = 0.15)
Pollen.ClusteringTree <- HGC.dendrogram(G = Pollen.SNN)

## -----------------------------------------------------------------------------
cluster.k5 <- cutree(Pollen.ClusteringTree, k = 5)

## ----eval = FALSE-------------------------------------------------------------
# library(dplyr)
# library(Seurat)
# library(patchwork)
# library(HGC)
# 
# # Load the PBMC dataset
# pbmc.data <- Read10X(data.dir =
#                 "../data/pbmc3k/filtered_gene_bc_matrices/hg19/")
# # Initialize the Seurat object with the raw (non-normalized data).
# pbmc <- CreateSeuratObject(counts = pbmc.data, project = "pbmc3k",
#                             min.cells = 3, min.features = 200)
# 
# # QC and selecting cells for further analysis
# pbmc[["percent.mt"]] <- PercentageFeatureSet(pbmc, pattern = "^MT-")
# pbmc <- subset(pbmc, subset = nFeature_RNA > 200 &
#                 nFeature_RNA < 2500 & percent.mt < 5)
# 
# # Normalizing the data
# pbmc <- NormalizeData(pbmc, normalization.method = "LogNormalize",
#                         scale.factor = 10000)
# 
# # Identification of highly variable features (feature selection)
# pbmc <- FindVariableFeatures(pbmc, selection.method = "vst",
#                                 nfeatures = 2000)
# 
# # Scaling the data
# all.genes <- rownames(pbmc)
# pbmc <- ScaleData(pbmc, features = all.genes)
# 
# # Perform linear dimensional reduction
# pbmc <- RunPCA(pbmc, features = VariableFeatures(object = pbmc))
# 
# # Determine the ‘dimensionality’ of the dataset
# pbmc <- JackStraw(pbmc, num.replicate = 100)
# pbmc <- ScoreJackStraw(pbmc, dims = 1:20)
# 
# # Construct the graph and cluster the cells with HGC
# pbmc <- FindNeighbors(pbmc, dims = 1:10)
# pbmc <- FindClusteringTree(pbmc, graph.type = "SNN")
# 
# # Output the tree
# pbmc.tree <- pbmc@graphs$ClusteringTree

## ----eval = FALSE-------------------------------------------------------------
# # Setting up the data
# library(scRNAseq)
# sce <- GrunPancreasData()
# 
# library(scuttle)
# qcstats <- perCellQCMetrics(sce)
# qcfilter <- quickPerCellQC(qcstats,
#                             percent_subsets="altexps_ERCC_percent")
# sce <- sce[,!qcfilter$discard]
# 
# library(scran)
# clusters <- quickCluster(sce)
# sce <- computeSumFactors(sce, clusters=clusters)
# sce <- logNormCounts(sce)
# 
# # Variance modelling
# dec <- modelGeneVar(sce)
# plot(dec$mean, dec$total, xlab="Mean log-expression",
#         ylab="Variance")
# curve(metadata(dec)$trend(x), col="blue", add=TRUE)
# 
# # Get the top 10% of genes.
# top.hvgs <- getTopHVGs(dec, prop=0.1)
# 
# sce <- fixedPCA(sce, subset.row=top.hvgs)
# reducedDimNames(sce)
# 
# # Automated PC choice
# output <- getClusteredPCs(reducedDim(sce))
# npcs <- metadata(output)$chosen
# reducedDim(sce, "PCAsub") <-
#     reducedDim(sce, "PCA")[,1:npcs,drop=FALSE]
# 
# 
# library(HGC)
# # Graph construction
# g <- buildSNNGraph(sce, use.dimred="PCAsub")
# # Graph-based clustering
# cluster.tree <- HGC.dendrogram(G = g)
# cluster.k12 <- cutree(cluster.tree, k = 12)
# 
# colLabels(sce) <- factor(cluster.k12)
# 
# library(scater)
# sce <- runTSNE(sce, dimred="PCAsub")
# plotTSNE(sce, colour_by="label", text_by="label")

## ----fig.height = 4.5---------------------------------------------------------
Pollen.ClusteringTree$height = log(Pollen.ClusteringTree$height + 1)
Pollen.ClusteringTree$height = log(Pollen.ClusteringTree$height + 1)

HGC.PlotDendrogram(tree = Pollen.ClusteringTree,
                    k = 5, plot.label = FALSE)

## ----fig.height = 4.5---------------------------------------------------------
Pollen.labels <- data.frame(Tissue = Pollen.Label.Tissue,
                            CellLine = Pollen.Label.CellLine)
HGC.PlotDendrogram(tree = Pollen.ClusteringTree,
                    k = 5, plot.label = TRUE, 
                    labels = Pollen.labels)

## -----------------------------------------------------------------------------
ARI.mat <- HGC.PlotARIs(tree = Pollen.ClusteringTree,
                        labels = Pollen.labels)

## -----------------------------------------------------------------------------
Pollen.ParameterRecord <- HGC.parameter(G = Pollen.SNN)

HGC.PlotParameter(Pollen.ParameterRecord, parameter = "CL")
HGC.PlotParameter(Pollen.ParameterRecord, parameter = "ANN")

# The function is similar to the dendrogram in "scipy.cluster.hierarchy".
# The function output the order of leaves in the clustering tree.

# The function is not open to users.

get.LeavesOrder <- function(linkageMatrix){
    return(get_leaves(linkageMatrix))
}

# Check the input G as dgCMatrix
# The function is not open to users.

checkGraph <- function(G){
    if(class(G)[1] != "dgCMatrix"){
        stop("Wrong graph data structure")
    }
    if(nrow(G) != ncol(G)){
        stop("Wrong graph data structure")
    }
    if(min(G) < 0){
        stop("Wrong graph data structure")
    }
    if(sum(is.na(G)) > 0){
        stop("Wrong graph data structure")
    }
    if(sum(is.infinite(G)) > 0){
        stop("Wrong graph data structure")
    }
}

# Check the input integer parameter
# The function is not open to users.

checkInteger <- function(t.parameter, str = "k"){
    if(t.parameter < 1 || round(t.parameter) != t.parameter){
        order <- paste(str, " should be positive integer.",
                        sep = "")
        stop(order)
    }
}

# check the input data structure
# The function is not open to users.

checkDataStructure <- function(object,
                                object.name,
                                expected.class = "matrix"){
    if(class(object)[1] != expected.class){
        order <- paste("Wrong input data structure for ",
                        object.name, ".", sep = "")
        stop(order)
    }
}

# HGC.time is a function to observe the nearest neighbor chain length and the
# average neighbor number during the clustering.

HGC.parameter <- function(G){
    # Run clustering
    checkGraph(G)
    record = HierarCluster_paris_time(G)
    rownames(record) = c("ChainLength", "AvgNeighNum")

    return(record)
}

set.seed(2000)
ncells <- 100
nPCs <- 20
simdata <- matrix(rnorm(nPCs*ncells), ncol=nPCs, nrow=ncells)

sim.SNN <- SNN.Construction(simdata)

# Good Input
test_that("The function could record the parameters", {
    expect_snapshot(HGC.parameter(G = sim.SNN))
})

# Bad Input
test_that("Wrong input for G",{
    expect_error(HGC.parameter(G = 552), "Wrong graph data structure")
    expect_error(HGC.parameter(G = simdata), "Wrong graph data structure")
    expect_error(HGC.parameter(G = sim.SNN[1:50,]), "Wrong graph data structure")
    expect_error(HGC.parameter(G = sim.SNN - 1), "Wrong graph data structure")
})

set.seed(2000)
ncells <- 100
nPCs <- 20
simdata <- matrix(rnorm(nPCs*ncells), ncol=nPCs, nrow=ncells)

sim.SNN <- SNN.Construction(simdata)
sim.ParameterRecord <- HGC.parameter(G = sim.SNN)

# Good Input
test_that("The function could show the figure", {
    expect_equal(HGC.PlotParameter(record = sim.ParameterRecord, parameter = "CL"), 1)
    expect_equal(HGC.PlotParameter(record = sim.ParameterRecord, parameter = "ANN"), 1)
})

# Bad Input: record
test_that("Wrong input for record",{
    expect_error(HGC.PlotParameter(record = 123, parameter = "CL"), "Wrong input data structure for record.")
    expect_error(HGC.PlotParameter(record = simdata, parameter = "CL"), "Wrong input data structure for record.")
    expect_error(HGC.PlotParameter(record = sim.SNN, parameter = "CL"), "Wrong input data structure for record.")
})

# Bad input: parameter
test_that("Wrong input for parameter",{
    expect_error(HGC.PlotParameter(record = sim.ParameterRecord, parameter = "XXX"), "Wrong parameter name.")
})

# Some published graph construction methods

matrix2dgCMatrix <- function(m){
    numrow <- nrow(m)
    numcolumn <- ncol(m)
    new.dgC <- Matrix::sparseMatrix(i = rep(seq_len(numrow), times = numcolumn),
                                    j = rep(seq_len(numcolumn), each = numrow),
                                    x = as.vector(m), dims =
                                    c(numrow, numcolumn))
    return(new.dgC)
}

KNN.Construction <- function(mat, k = 20){
    checkInteger(k, "k")
    ncell <- nrow(mat)
    nfeature <- ncol(mat)
    if(k >= ncell){
        stop("k should be smaller than cell number.")
    }
    tmp <- RANN::nn2(mat, mat, k + 1, searchtype = "standard")
    neighborMatrix <- tmp[[1]][, -1]
    distMatrix <- tmp[[2]][, -1]
    knn.G <- Matrix::sparseMatrix(i = rep(seq_len(ncell), k),
                                j = as.vector(neighborMatrix),
                                x = 1, dims = c(ncell, ncell))
    knn.G <- (knn.G + Matrix::t(knn.G))/2
    knn.G[Matrix::which(knn.G>0)] = 1
    #knn.G <- as.matrix(G)

    return(knn.G)
}

RNN.Construction <- function(mat, max_dist){
    if(max_dist < 0){
        stop("The threshold distance should be non-negative.")
    }
    ncell <- nrow(mat)
    nfeature <- ncol(mat)
    D <- stats::dist(mat)
    G <- as.matrix(D)
    G[which(G>max_dist)] <- 0
    G[which(G>0)] <- 1

    #rnn.G <- Matrix::Matrix(G, sparse = TRUE)
    rnn.G <- matrix2dgCMatrix(G)
    return(rnn.G)
}

CKNN.Construction <- function(mat, k = 20, delta = 1){
    checkInteger(k, "k")
    ncell <- nrow(mat)
    nfeature <- ncol(mat)
    if(k >= ncell){
        stop("k should be smaller than cell number.")
    }
    if(delta <= 0){
        stop("The parameter delta should be positive.")
    }

    Dist_original <- as.matrix(stats::dist(mat))
    ## Build the knn on the original space
    tmp <- RANN::nn2(mat, mat, k + 1, searchtype = "standard")
    neighborMatrix <- tmp[[1]][, -1]
    distMatrix <- tmp[[2]][, -1]

    cknn.G <- matrix(0, nrow = ncell, ncol = ncell)
    for(i in seq_len(ncell-1)){
        for(j in seq(i+1,ncell)){
            distk_i <- distMatrix[i,k]
            distk_j <- distMatrix[j,k]
            if(Dist_original[i,j] < delta*sqrt(distk_i*distk_j)){
                cknn.G[i,j] = 1
                cknn.G[j,i] = 1
            }
        }
    }

    cknn.G <- matrix2dgCMatrix(cknn.G)
    return(cknn.G)
}

SNN.Construction <- function(mat, k = 20, threshold = 1/15){
    checkInteger(k, "k")
    ncell <- nrow(mat)
    nfeature <- ncol(mat)
    if(k >= ncell){
        stop("k should be smaller than cell number.")
    }
    if(threshold < 0 || threshold > 1){
        stop("The parameter threshold should be within zero to one.")
    }

    tmp <- RANN::nn2(mat, mat, k + 1, searchtype = "standard")
    neighborMatrix <- tmp[[1]][, -1]
    snn.G <- ComputeSNN(nn_ranked = neighborMatrix,
                        prune = threshold)

    return(snn.G)
}

MST.Construction <- function(mat){
    ncell <- nrow(mat)
    nfeature <- ncol(mat)
    D <- stats::dist(mat)
    dmst <- ape::mst(D)

    mst.G <- matrix(0, nrow = ncell, ncol = ncell)
    mst.G[which(dmst > 0)] = 1

    mst.G <- matrix2dgCMatrix(mst.G)
    return(mst.G)
}

PMST.Construction <- function(mat, iter = 20, r = 0.5){
    checkInteger(iter, "iter")
    if(r <= 0){
        stop("The parameter r should be positive.")
    }
    ncell <- nrow(mat)
    nfeature <- ncol(mat)
    D <- stats::dist(mat)
    mst.G <- matrix(0, nrow = ncell, ncol = ncell)
    ## Build the knn on the original space
    k <- 3
    tmp <- RANN::nn2(mat, mat, k + 1, searchtype = "standard")
    neighborMatrix <- tmp[[1]][, -1]
    distMatrix <- tmp[[2]][, -1]
    # for each node, record the coordinate of the
    # average of its three nearest neighbors
    neighborLocation <- mat
    for(i in seq_len(ncell)){
        neighbor1 <- neighborMatrix[i,1]
        neighbor2 <- neighborMatrix[i,2]
        neighbor3 <- neighborMatrix[i,3]
        neighbor1.site <- as.vector(mat[neighbor1,])
        neighbor2.site <- as.vector(mat[neighbor2,])
        neighbor3.site <- as.vector(mat[neighbor3,])
        new.neighbor <- (neighbor1.site +
            neighbor2.site + neighbor3.site)/3
        neighborLocation[i,] <- new.neighbor
    }
    ## Perturbation
    for(i in seq_len(iter)){
        new.mat1 <- mat
        new.mat2 <- neighborLocation

        random.r1 <- stats::runif(n = ncell, min = 0, max = r)
        random.r2 <- rep(1, ncell) - random.r1

        new.mat <- new.mat1*random.r1 + new.mat2*random.r2
        new.D <- stats::dist(new.mat)
        new.mst <- ape::mst(new.D)

        new.mst.G <- matrix(0, nrow = ncell, ncol = ncell)
        new.mst.G[which(new.mst > 0)] = 1

        mst.G <- mst.G + new.mst.G
    }
    mst.G[which(mst.G > 0)] = 1
    mst.G <- matrix2dgCMatrix(mst.G)
    return(mst.G)
}

set.seed(2000)
ncells <- 100
nPCs <- 20
simdata <- matrix(rnorm(nPCs*ncells), ncol=nPCs, nrow=ncells)

# Good Input
test_that("The function could record the parameters", {
    expect_snapshot(CKNN.Construction(simdata, 20, 1))
    expect_s4_class(CKNN.Construction(simdata, 20, 1), "dgCMatrix")
})

# Bad Input: k
test_that("Wrong input for k",{
    expect_error(CKNN.Construction(simdata, -1, 1), "k should be positive integer.")
    expect_error(CKNN.Construction(simdata, 0, 1), "k should be positive integer.")
    expect_error(CKNN.Construction(simdata, 2.5, 1), "k should be positive integer.")
    expect_error(CKNN.Construction(simdata, 10000, 1), "k should be smaller than cell number.")
})

# Bad Input: delta
test_that("Wrong input for delta",{
    expect_error(CKNN.Construction(simdata, 20, -1), "The parameter delta should be positive.")
})

set.seed(2000)
ncells <- 100
nPCs <- 20
simdata <- matrix(rnorm(nPCs*ncells), ncol=nPCs, nrow=ncells)

# Good Input
test_that("The function could record the parameters", {
    expect_snapshot(MST.Construction(simdata))
    expect_s4_class(MST.Construction(simdata), "dgCMatrix")
})

set.seed(2000)
ncells <- 100
nPCs <- 20
simdata <- matrix(rnorm(nPCs*ncells), ncol=nPCs, nrow=ncells)

sim.SNN <- SNN.Construction(simdata)

# Good Input
test_that("The function could record the parameters", {
    expect_snapshot(HGC.dendrogram(G = sim.SNN))
    expect_s3_class(HGC.dendrogram(G = sim.SNN), "hclust")
})

# Bad Input
test_that("Wrong input for G",{
  expect_error(HGC.dendrogram(G = 552), "Wrong graph data structure")
  expect_error(HGC.dendrogram(G = simdata), "Wrong graph data structure")
  expect_error(HGC.dendrogram(G = sim.SNN[1:50,]), "Wrong graph data structure")
  expect_error(HGC.dendrogram(G = sim.SNN - 1), "Wrong graph data structure")
})

set.seed(2000)
ncells <- 100
nPCs <- 20
simdata <- matrix(rnorm(nPCs*ncells), ncol=nPCs, nrow=ncells)

# Good Input
test_that("The function could record the parameters", {
    expect_snapshot(KNN.Construction(simdata, 20))
    expect_s4_class(KNN.Construction(simdata, 20), "dgCMatrix")
})

# Bad Input
test_that("Wrong input for k",{
    expect_error(KNN.Construction(simdata, -1), "k should be positive integer.")
    expect_error(KNN.Construction(simdata, 0), "k should be positive integer.")
    expect_error(KNN.Construction(simdata, 2.5), "k should be positive integer.")
    expect_error(KNN.Construction(simdata, 10000), "k should be smaller than cell number.")
})

set.seed(2000)
ncells <- 100
nPCs <- 20
simdata <- matrix(rnorm(nPCs*ncells), ncol=nPCs, nrow=ncells)

# Good Input
test_that("The function could record the parameters", {
  expect_snapshot(RNN.Construction(simdata, 2.5))
  expect_s4_class(RNN.Construction(simdata, 2.5), "dgCMatrix")
})

# Bad Input
test_that("Wrong input for dist",{
  expect_error(RNN.Construction(simdata, -1), "The threshold distance should be non-negative.")
})

set.seed(2000)
ncells <- 100
nPCs <- 20
simdata <- matrix(rnorm(nPCs*ncells), ncol=nPCs, nrow=ncells)

# Good Input
test_that("The function could record the parameters", {
    expect_snapshot(PMST.Construction(simdata, 20, 0.5))
    expect_s4_class(PMST.Construction(simdata, 20, 0.5), "dgCMatrix")
})

# Bad Input: iter
test_that("Wrong input for iter",{
    expect_error(PMST.Construction(simdata, -1, 0.5), "iter should be positive integer.")
    expect_error(PMST.Construction(simdata, 0, 0.5), "iter should be positive integer.")
    expect_error(PMST.Construction(simdata, 2.5, 0.5), "iter should be positive integer.")
})

# Bad Input: r
test_that("Wrong input for r",{
    expect_error(PMST.Construction(simdata, 20, -0.5), "The parameter r should be positive.")
})

set.seed(2000)
ncells <- 100
nPCs <- 20
simdata <- matrix(rnorm(nPCs*ncells), ncol=nPCs, nrow=ncells)

sim.SNN <- SNN.Construction(simdata)
sim.ClusteringTree <- HGC.dendrogram(G = sim.SNN)

# Good Input
sim.labels <- data.frame(Tissue = sample(c("A","B"), 100, replace = TRUE),
                            CellLine = sample(c("AA","BA", "BB"), 100, replace = TRUE))

test_that("The function could show the figure", {
  expect_equal(HGC.PlotARIs(tree = sim.ClusteringTree, k.min = 2, k.max = 15, sim.labels), 1)
  expect_equal(HGC.PlotARIs(tree = sim.ClusteringTree, k.min = 5, k.max = 15, sim.labels), 1)
  expect_equal(HGC.PlotARIs(tree = sim.ClusteringTree, k.min = 2, k.max = 25, sim.labels), 1)
  expect_equal(HGC.PlotARIs(tree = sim.ClusteringTree, k.min = 5, k.max = 25, sim.labels), 1)
})

# Bad Input: tree
test_that("Wrong input for tree",{
    expect_error(HGC.PlotARIs(tree = 1, k.min = 2, k.max = 15, sim.labels), "Wrong input data structure for tree.")
    expect_error(HGC.PlotARIs(tree = simdata, k.min = 2, k.max = 15, sim.labels), "Wrong input data structure for tree.")
})

# Bad input: k
test_that("Wrong input for k.min or k.max",{
  expect_error(HGC.PlotARIs(tree = sim.ClusteringTree, k.min = -1, k.max = 15, sim.labels), "k.min and k.max should be positive integer.")
  expect_error(HGC.PlotARIs(tree = sim.ClusteringTree, k.min = 1.1, k.max = 15, sim.labels), "k.min and k.max should be positive integer.")
  expect_error(HGC.PlotARIs(tree = sim.ClusteringTree, k.min = 2, k.max = 15.1, sim.labels), "k.min and k.max should be positive integer.")
  expect_error(HGC.PlotARIs(tree = sim.ClusteringTree, k.min = 2, k.max = 10000, sim.labels), "k.min and k.max should be smaller than cell number.")
  expect_error(HGC.PlotARIs(tree = sim.ClusteringTree, k.min = 15, k.max = 2, sim.labels), "k.min should not be larger than k.max.")
})

# Bad input: labels
test_that("Wrong input for labels",{
  expect_error(HGC.PlotARIs(tree = sim.ClusteringTree, k.min = 2, k.max = 15, sim.labels[-1,]), "The labels do not match the data.")
  expect_error(HGC.PlotARIs(tree = sim.ClusteringTree, k.min = 2, k.max = 15, rbind(sim.labels,sim.labels)), "The labels do not match the data.")
})

set.seed(2000)
ncells <- 100
nPCs <- 20
simdata <- matrix(rnorm(nPCs*ncells), ncol=nPCs, nrow=ncells)

# Good Input
test_that("The function could record the parameters", {
    expect_snapshot(SNN.Construction(simdata, 20, 1/15))
    expect_s4_class(SNN.Construction(simdata, 20, 1/15), "dgCMatrix")
})

# Bad Input: k
test_that("Wrong input for k",{
    expect_error(SNN.Construction(simdata, -1, 1/15), "k should be positive integer.")
    expect_error(SNN.Construction(simdata, 0, 1/15), "k should be positive integer.")
    expect_error(SNN.Construction(simdata, 2.5, 1/15), "k should be positive integer.")
    expect_error(SNN.Construction(simdata, 10000, 1/15), "k should be smaller than cell number.")
})

# Bad Input: threshold
test_that("Wrong input for threshold",{
    expect_error(SNN.Construction(simdata, 20, -1), "The parameter threshold should be within zero to one.")
    expect_error(SNN.Construction(simdata, 20, 2), "The parameter threshold should be within zero to one.")
})

set.seed(2000)
ncells <- 100
nPCs <- 20
simdata <- matrix(rnorm(nPCs*ncells), ncol=nPCs, nrow=ncells)

sim.SNN <- SNN.Construction(simdata)
sim.ClusteringTree <- HGC.dendrogram(G = sim.SNN)

# Good Input
sim.labels <- data.frame(Tissue = sample(c("A","B"), 100, replace = TRUE),
                            CellLine = sample(c("AA","BA", "BB"), 100, replace = TRUE))

test_that("The function could show the figure", {
    expect_equal(HGC.PlotDendrogram(tree = sim.ClusteringTree, k = 5, plot.label = FALSE), 1)
    expect_equal(HGC.PlotDendrogram(tree = sim.ClusteringTree, k = 10, plot.label = FALSE), 1)
    expect_equal(HGC.PlotDendrogram(tree = sim.ClusteringTree, k = 50, plot.label = FALSE), 1)
    expect_equal(HGC.PlotDendrogram(tree = sim.ClusteringTree, k = 2, plot.label = TRUE, labels = sim.labels), 1)
    expect_equal(HGC.PlotDendrogram(tree = sim.ClusteringTree, k = 5, plot.label = TRUE, labels = sim.labels), 1)
    expect_equal(HGC.PlotDendrogram(tree = sim.ClusteringTree, k = 10, plot.label = TRUE, labels = sim.labels), 1)
    expect_equal(HGC.PlotDendrogram(tree = sim.ClusteringTree, k = 50, plot.label = TRUE, labels = sim.labels), 1)
})

# Bad Input: tree
test_that("Wrong input for tree",{
    expect_error(HGC.PlotDendrogram(tree = 1, k = 2, plot.label = FALSE), "Wrong input data structure for tree.")
    expect_error(HGC.PlotDendrogram(tree = simdata, k = 2, plot.label = FALSE), "Wrong input data structure for tree.")
})

# Bad input: k
test_that("Wrong input for k",{
    expect_error(HGC.PlotDendrogram(tree = sim.ClusteringTree, k = -1, plot.label = FALSE), "k should be positive integer.")
    expect_error(HGC.PlotDendrogram(tree = sim.ClusteringTree, k = 0, plot.label = FALSE), "k should be positive integer.")
    expect_error(HGC.PlotDendrogram(tree = sim.ClusteringTree, k = 1.01, plot.label = FALSE), "k should be positive integer.")
    expect_error(HGC.PlotDendrogram(tree = sim.ClusteringTree, k = 100000, plot.label = FALSE), "k should be smaller than cell number.")
})

# Bad input: labels
test_that("Wrong input for labels",{
  expect_error(HGC.PlotDendrogram(tree = sim.ClusteringTree, k = 2, plot.label = TRUE), "argument \"labels\" is missing, with no default")
  expect_error(HGC.PlotDendrogram(tree = sim.ClusteringTree, k = 2, plot.label = TRUE, labels = sim.labels[1:5,]), "The labels do not match the data.")
})

# HGC.paris is the main function to cluster. It outputs the clustering tree.

HGC.dendrogram <- function(G){
    UseMethod("HGC.dendrogram")
}

HGC.dendrogram.default <- function(G){
    # Run clustering
    checkGraph(G)
    hg.linkageMatrix = HierarCluster_paris(G)

    # check the linkageMatrix
    hg.linkageMatrix2 <- hg.linkageMatrix
    hg.linkageMatrix.temp <- as.vector(hg.linkageMatrix[,3])

    # The unconnected branches problem
    link.na <- which(is.infinite(hg.linkageMatrix.temp))
    if(length(link.na) > 0){
        hg.linkageMatrix.temp <- hg.linkageMatrix.temp[-link.na]
        hg.linkageMatrix2[which(is.infinite(hg.linkageMatrix))] <- 10*max(
            hg.linkageMatrix.temp, na.rm = TRUE)
    }

    hg.tree2 = list()
    class(hg.tree2) <- 'hclust'
    hg.tree2$call = NA
    hg.tree2$dist.method = "Node pair sampling ratio"
    hg.tree2$method = "HGC"

    hg.tree2$height = hg.linkageMatrix2[,3]

    N = ncol(G)
    merge.m = hg.linkageMatrix2[,seq_len(2)] + 1
    merge.m[merge.m<=N] = merge.m[merge.m<=N]*-1
    merge.m[merge.m>N] = merge.m[merge.m>N]-N

    hg.tree2$merge = merge.m

    hg.tree2$order = get.LeavesOrder(hg.linkageMatrix2) + 1

    if(length(rownames(G)) == nrow(G)){
        hg.tree2$labels = rownames(G)
    }else{
        hg.tree2$labels = NA
    }

    return(hg.tree2)
}

HGC.dendrogram.dgCMatrix <- function(G){
    return(HGC.dendrogram.default(G))
}

HGC.dendrogram.graph <- function(G){
    class(G) <- "dgCMatrix"
    return(HGC.dendrogram.default(G))
}

HGC.dendrogram.matrix <- function(G){
    G <- as(G, "dgCMatrix")
    return(HGC.dendrogram.default(G))    
}

HGC.dendrogram.igraph <- function(G){
    require(igraph)
    G <- igraph::as_adjacency_matrix(G, sparse = TRUE)
    return(HGC.dendrogram.default(G))
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

HierarCluster_paris <- function(m) {
    .Call(`_HGC_HierarCluster_paris`, m)
}

HierarCluster_paris_time <- function(m) {
    .Call(`_HGC_HierarCluster_paris_time`, m)
}

get_leaves <- function(hglink) {
    .Call(`_HGC_get_leaves`, hglink)
}

ComputeSNN <- function(nn_ranked, prune) {
    .Call(`_HGC_ComputeSNN`, nn_ranked, prune)
}

WriteEdgeFile <- function(snn, filename, display_progress) {
    invisible(.Call(`_HGC_WriteEdgeFile`, snn, filename, display_progress))
}

DirectSNNToFile <- function(nn_ranked, prune, display_progress, filename) {
    .Call(`_HGC_DirectSNNToFile`, nn_ranked, prune, display_progress, filename)
}

SNN_SmallestNonzero_Dist <- function(snn, mat, n, nearest_dist) {
    .Call(`_HGC_SNN_SmallestNonzero_Dist`, snn, mat, n, nearest_dist)
}

gg_color_ggplot <- function(n){
    hues = seq(15, 375, length = n + 1)
    grDevices::hcl(h = hues, l = 65, c = 100)[seq_len(n)]
}

HGC.PlotDendrogram <- function(tree, k = 5, plot.label = FALSE, labels){
    checkDataStructure(tree, "tree", "hclust")
    ncell <- length(tree$order)
    if(ncell == 0){
        stop("There is no element in the tree.")
    }
    checkInteger(k, "k")
    if(k > ncell){
        stop("k should be smaller than cell number.")
    }
    if(plot.label){
        if(nrow(labels) != ncell){
            stop("The labels do not match the data.")
        }
    }
    clus.dendrogram <- stats::as.dendrogram(tree)
    dend <- clus.dendrogram %>%
            dendextend::set("branches_k_color", k=k) %>%
            dendextend::set("branches_lwd", 1.2) %>%
            dendextend::set("labels_colors", "white")
    if(plot.label == FALSE){
        plot(dend)
        return(1)
    }else{
        n_label <- ncol(labels)
        n_diff_labels <- rep(0, n_label)
        for(i in seq_len(n_label)){
            temp_n_label <- length(unique(labels[,i]))
            n_diff_labels[i] <- temp_n_label
        }
        T.colourbar <- matrix(nrow = ncell, ncol = n_label)
        colnames(T.colourbar) = colnames(labels)
        for(i in seq_len(n_label)){
            temp_label <- labels[,i]
            temp_u_label <- unique(labels[,i])
            temp_n_label <- n_diff_labels[i]
            temp_colourbar <- gg_color_ggplot(temp_n_label)
            for(j in seq_len(temp_n_label)){
                T.colourbar[which(temp_label ==
                                temp_u_label[j]),i] = temp_colourbar[j]
            }
        }
        plot(dend)
        dendextend::colored_bars(colors = T.colourbar[
            order.dendrogram(dend),])
        return(1)
    }
}

HGC.PlotARIs <- function(tree, k.min = 2, k.max = 15,
                        labels, return.ARI = FALSE){
    n_label <- ncol(labels)
    checkDataStructure(tree, "tree", "hclust")
    ncell <- length(tree$order)
    if(ncell == 0){
        stop("There is no element in the tree.")
    }
    if(k.min < 1 || round(k.min) != k.min || k.max < 1 ||
        round(k.max) != k.max){
        stop("k.min and k.max should be positive integer.")
    }
    if(k.min > ncell || k.max > ncell){
        stop("k.min and k.max should be smaller than cell number.")
    }
    if(k.min > k.max){
        stop("k.min should not be larger than k.max.")
    }
    if(nrow(labels) != ncell){
        stop("The labels do not match the data.")
    }
    hc.ari.mat1 <- matrix(nrow = k.max - k.min + 1, ncol = n_label)
    colnames(hc.ari.mat1) <- colnames(labels)
    for(j in seq_len(n_label)){
        temp_label <- labels[,j]
        for(i in seq(k.min, k.max)){
            hc.ari.mat1[i-k.min+1,j] = mclust::adjustedRandIndex(
                                        temp_label, cutree(tree, i))
        }
    }
    # HC line chart
    plot.df2 <- data.frame(k = factor(rep(seq(k.min, k.max), times = n_label)),
                            ARI = as.vector(hc.ari.mat1),
                            label = c(rep(colnames(hc.ari.mat1), each =
                                        k.max - k.min + 1)))

    p1 <- ggplot2::ggplot(plot.df2, ggplot2::aes_string(x="k", y="ARI",
                                        group="label", colour="label")) +
        ggplot2::geom_line() +
        ggplot2::geom_point(size=1) +
        ggplot2::theme(legend.title = ggplot2::element_blank())
    plot(p1)
    if(return.ARI){
        return(hc.ari.mat1)
    }else{
        return(1)
    }
}

HGC.PlotParameter <- function(record, parameter = "CL"){
    n_iter <- ncol(record)
    checkDataStructure(record, "record", "matrix")
    if(nrow(record) != 2){
        stop("Wrong input data structure for record.")
    }
    if(rownames(record)[1] != "ChainLength" ||
        rownames(record)[2] != "AvgNeighNum"){
        stop("Wrong input data structure for record.")
    }
    if(parameter == "CL"){
        plot.df <- data.frame(Iter = seq_len(n_iter), ChainLength = record[1,])

        p1 <- ggplot2::ggplot(plot.df,
                            ggplot2::aes_string(x="Iter", y="ChainLength")) +
            ggplot2::geom_point(colour = "#00BFC4", size = 1.0)
        p2_hist <- ggplot2::ggplot(plot.df,
                            ggplot2::aes_string(x="ChainLength")) +
            ggplot2::geom_histogram(binwidth = 1,
                                    colour = "black",
                                    fill = "#00BF7D") +
            ggplot2::coord_flip() +
            ggplot2::theme_void()
        plot(patchwork::wrap_plots(p1, p2_hist,
                                    nrow = 1, widths = c(1, 0.5)))
        return(1)
    }else if(parameter == "ANN"){
        plot.df <- data.frame(Iter = seq_len(n_iter), ANN = record[2,])

        p1 <- ggplot2::ggplot(plot.df,
                            ggplot2::aes_string(x="Iter", y="ANN")) +
            ggplot2::geom_point(colour = "#F8766D", size = 1.0)
        p2_hist <- ggplot2::ggplot(plot.df,
                            ggplot2::aes_string(x="ANN")) +
            ggplot2::geom_histogram(binwidth = 1,
                                    colour = "black",
                                    fill = "#A3A500") +
            ggplot2::coord_flip() +
            ggplot2::theme_void()
        plot(patchwork::wrap_plots(p1, p2_hist,
                                    nrow = 1, widths = c(1, 0.5)))
        return(1)
    }else{
        stop("Wrong parameter name.")
    }
}

# The function could record the parameters

    Code
      HGC.dendrogram(G = sim.SNN)
    Output
      
      Call:
      NA
      
      Cluster method   : HGC 
      Distance         : Node pair sampling ratio 
      Number of objects: 100 
      

\name{HGC.PlotDendrogram}
\alias{HGC.PlotDendrogram}
\title{
Visualizing the dendrogram
}
\description{
The function will plot the dendrogram, with
different colors for different clusters.
}
\usage{
HGC.PlotDendrogram(tree, k, plot.label, labels)
}
\arguments{
    \item{tree}{
    the input clustering tree saved as \code{hclust} data structure.
    }
    \item{k}{
    the number of clusters to cut the tree into.
    }
    \item{plot.label}{
    a bool variable. It decides whether the function will add color bars.
    }
    \item{labels}{
    a data frame or a matrix to store the label information.
    Different labels should be in different columns and the
    users should name the columns correspondingly. The label
    information will show in the figure as color
    bars below the dendrogram, and each label takes one color bar.
    }
}
\details{
The function plots the clustering tree, with alternative colors showing
the clustering results and the label information. It is based on the R
package \code{dendextend} which contains many parameters for the
visualization. For users' convenience, most of the parameters are set to
be the default value. Advanced users could
visit the vignette of \code{dendextend} for more flexible visualization.
}
\value{
The function will return 1 if the dendrogram is successfully drawn.
}
\examples{
data(Pollen)
Pollen.PCs <- Pollen[["PCs"]]
Pollen.Label.Tissue <- Pollen[["Tissue"]]
Pollen.Label.CellLine <- Pollen[["CellLine"]]

Pollen.SNN <- SNN.Construction(Pollen.PCs)
Pollen.ClusteringTree <- HGC.dendrogram(G = Pollen.SNN)
Pollen.labels <- data.frame(Tissue = Pollen.Label.Tissue,
                            CellLine = Pollen.Label.CellLine)
HGC.PlotDendrogram(tree = Pollen.ClusteringTree,
                    k = 5, plot.label = TRUE,
                    labels = Pollen.labels)
}

\name{HGC.dendrogram}
\alias{HGC.dendrogram}
\title{
Hierarchical Graph-based Clustering
}
\description{
Hierarchical clustering on a given undirected graph.
}
\usage{
HGC.dendrogram(G)
}
\arguments{
    \item{G}{
    an object which represents the adjacency matrix of the graph, 
    where G[i,j] is the weight
    of the edge between node i and node j, and zero means
    no link.
    
    The supported data structures include \code{matrix}, 
    \code{dgCMatrix}, \code{graph}, and \code{igraph}.
    }
}
\details{
The function runs a hierarchical clustering on the given graph. It is a
recursive procedure of two steps, first, the node
pair sampling ratio is used as the distance metric to search the nearest
neighbor pairs. Then the neighbor pair are merged and the graph is updated.
The whole procedure is accelerated using the nearest neighbor chain algorithm.
The algorithm stops when there's only one node left in the updated graph.
}
\value{
An object of class \code{hclust} defined by the \code{hclust} function
in the \code{stats} package. It is a list containing the clustering tree
information with the components:
\item{merge }{an \emph{n-1} by 2 matrix. It records the two nodes in
each merging step.}
\item{height }{a set of \emph{n-1} real values. It is the height of the
non-leaf nodes in the tree.}
\item{order }{a vector giving the permutation of the original
observations suitable for plotting.}
\item{labels }{labels for the objects being clustered. Same as the
rownames of G in default.}
\item{call }{the call which produced the result.}
\item{method }{the cluster method that has been used.}
\item{dist.method }{the distance used here.}

More details about the components are in the \link[stats]{hclust}.
}
\examples{
data(Pollen)
Pollen.PCs <- Pollen[["PCs"]]
G <- SNN.Construction(Pollen.PCs, 25, 0.15)
tree = HGC.dendrogram(G)
}

\docType{data}
\name{Pollen}
\alias{Pollen}
\title{Embeddings of the Pollen datasets in the principal component space.}
\format{
An object of class \code{list}.

The \code{list} contains three elements: First, a matrix with
301 rows and 25 columns, saved as \code{Pollen[["PCs"]]}. The
rows are cells and columns are principal components. Second,
the label in tissue level, saved as a vector in
\code{Pollen[["Tissue"]]}.
Third, the label in cell line level, saved as a vector in
\code{Pollen[["CellLine"]]}.
}
\source{
\url{https://www.nature.com/articles/nbt.2967}
}
\description{
The dataset is the low dimensional principal components and
labels of cells from the Pollen dataset. The 301 cells are from
11 different cell lines and are classified into 4 tissues.
}
\keyword{datasets}

\name{RNN.Construction}
\alias{RNN.Construction}
\title{
Building Unweighted \eqn{\epsilon} Nearest Neighbor Graph
}
\description{
This function builds an \eqn{\epsilon} Nearest Neighbor graph in the
input feature space using Euclidean distance metric.
}
\usage{
RNN.Construction(mat, max_dist)
}
\arguments{
    \item{mat}{
    the input data saved as a numerical matrix. The columns are the
    features and the rows are the samples.
    }
    \item{max_dist}{
    the threshold distance. The edges whose lengths are less than
    \code{max_dist} will be kept in the graph.
    }
}
\details{
The function builds an \eqn{\epsilon} Nearest Neighbor graph  which
saved as a sparse matrix.
}
\value{
An n by n binary dgCMatrix object C, where n is the number of input
samples.
The matrix C is the adjacency matrix of the built RNN graph. C[i,j] = 1
means that there is an edge between sample i and sample j.
}
\examples{
data(Pollen)
Pollen.PCs <- Pollen[["PCs"]]
G <- RNN.Construction(Pollen.PCs, 20)
}

\name{KNN.Construction}
\alias{KNN.Construction}
\title{
Building Unweighted K Nearest Neighbor Graph
}
\description{
This function builds an Unweighted K Nearest Neighbor (KNN) graph 
in the input feature space using Euclidean distance metric.
}
\usage{
KNN.Construction(mat, k)
}
\arguments{
    \item{mat}{
    the input data saved as a numerical matrix. The columns are the
    features and the rows are the samples.
    }
    \item{k}{
    the number of nearest neighbors for building the KNN graph.
    }
}
\details{
This function builds a KNN graph of the input data. The main function 
comes from the R package \code{RANN}.
}
\value{
An n by n binary dgCMatrix object C, where n is the number of input 
samples.
The matrix C is the adjacency matrix of the built KNN graph. C[i,j] = 1  
means that there is an edge between sample i and sample j.
}
\examples{
data(Pollen)
Pollen.PCs <- Pollen[["PCs"]]
G <- KNN.Construction(Pollen.PCs)
}

\name{FindClusteringTree}
\alias{FindClusteringTree}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
The HGC algorithm embedded in Seurat pipeline
}
\description{
The function runs hierarchical clustering with \code{HGC.dendrogram} 
on the SNN or KNN calculated by the \code{Seurat} 
pipeline. The output clustering tree is also packaged in the 
\code{Seurat} object.
}
\usage{
FindClusteringTree(object, graph.type)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
    \item{object}{
    The Seurat object containing the graphs built with scRNA-seq data.
}
    \item{graph.type}{
    The type of graphs used for the hierarchical clustering, could be 
    "SNN" or "KNN". The default value is "SNN".
}
}
\details{
For the KNN graph, we symmetrize it by adding its transposition on 
the graph. And for the details of data preprocessing and graph construction 
by \code{Seurat}, please check the \code{Seurat} vignettes.
}
\value{
An \code{Seurat} object. The clustering tree is saved under the item 
\code{graphs}, i.e. \code{object@graphs$ClusteringTree}.
}

\note{
The function needs the R package \code{Seurat}. We recommend that the 
version of \code{Seurat} is higher than version 3.0.
}

\examples{
## Do not run
# require(Seurat)

## DemoData is a input gene expression matrix.
# DemoData.seuratobj <- CreateSeuratObject(counts = DemoData,
#                                          min.cells = 20)
# DemoData.seuratobj <- NormalizeData(object = DemoData.seuratobj,
#                                     verbose = F)
# DemoData.seuratobj <- ScaleData(object = DemoData.seuratobj,
#                                 features = row.names(DemoData.seuratobj),
#                                 verbose = F)
# DemoData.seuratobj <- FindVariableFeatures(object = DemoData.seuratobj,
#                                            nfeatures = 2000, verbose = F)
# DemoData.seuratobj <- RunPCA(object = DemoData.seuratobj,
#                              npcs = 100, verbose = F)
# DemoData.seuratobj <- FindNeighbors(object = DemoData.seuratobj,
#                                     nn.eps = 0.5, k.param = 30,
#                                     dims = 1:25, verbose = F)
# DemoData.seuratobj <- FindClusteringTree(object = DemoData.seuratobj,
#                                          graph.type = "SNN")
}

\name{HGC.parameter}
\alias{HGC.parameter}
\title{
Recording the Parameters of the Graph-based Hierarchical Clustering
}
\description{
This function records and outputs the length of the nearest neighbor 
chain and the average neighbor
number in each iteration of hierarchical clustering. These values can 
be used for the time complexity analysis of \code{HGC.dendrogram}.
}
\usage{
HGC.parameter(G)
}
\arguments{
    \item{G}{
    an undirected graph saved as a dgCMatrix. The matrix G is the adjacency
    matrix of the graph, and element G[i,j] is the weight of the edge between
    node i and node j. Zeros in the matrix mean no link between nodes here.
    }
}
\details{
This function contains the whole function of \code{HGC.dendrogram}, but will 
record the key parameters during the whole process.  The function is provided 
for advanced users to conduct time complexity analysis on their own data. 
The construction of the dendrogram is a recursive procedure of two steps: 
1.finding the nearest neighbour pair, 2. merge the node pair and update the 
graph. For different data structures of graph, there's a trade-off between 
the time consumptions of the two steps. Generally speaking, storing more 
information about the graph makes it faster to find the nearest neighbour 
pair (step 1) but slower to update the graph (step 2). We have experimented 
serval datasets and chosen the best data structure in \code{HGC.dendrogram} 
for the overall efficiency.
}
\value{
A 2 by m matrix. The two rows of the matrix are the nearest neighbor chain 
length and the average neighbor number. m is equal to n-s, where s is 
the number of unconnected parts in the graph.
}
\examples{
data(Pollen)
Pollen.PCs <- Pollen[["PCs"]]
G <- SNN.Construction(Pollen.PCs, 25, 0.15)
record = HGC.parameter(G)
}

\name{MST.Construction}
\alias{MST.Construction}
\title{
Building Unweighted Minimum Spanning Tree Graph
}
\description{
This function builds an Unweighted Minimum Spanning Tree (MST) graph 
in the input feature space using Euclidean distance metric.
}
\usage{
MST.Construction(mat)
}
\arguments{
    \item{mat}{
    the input data saved as a numerical matrix. The columns are the
    features and the rows are the samples.
    }
}
\details{
This function builds a MST graph of the input data. The main
function come from the R package \code{ape}.
}
\value{
An n by n binary dgCMatrix object C, where n is the number of input samples.
The matrix C is the adjacency matrix of the built MST graph. C[i,j] = 1  
means that there is an edge between sample i and sample j.
}
\examples{
data(Pollen)
Pollen.PCs <- Pollen[["PCs"]]
G <- MST.Construction(Pollen.PCs)
}

\name{CKNN.Construction}
\alias{CKNN.Construction}
\title{
Building Unweighted Continuous K Nearest Neighbor Graph
}
\description{
This function builds a Continuous K Nearest Neighbor (CKNN) graph
in the input feature space using Euclidean distance metric.
}
\usage{
CKNN.Construction(mat, k, delta)
}
\arguments{
    \item{mat}{
    the input data saved as a numerical matrix. The columns are the
    features and the rows are the samples.
    }
    \item{k}{
    the number of nearest neighbors for building the CKNN graph.
    }
    \item{delta}{
    the parameter related with the distance threshold.
    }
}
\details{
This function fist built a KNN graph from the input data. Then the CKNN
graph is built from the KNN graph. For node i and node j in
the KNN graph, CKNN will link them if the distance d(i,j) between node
i and node j is less than \eqn{\delta} times of the geometric mean of
d_k(i) and d_k(j). Here \eqn{\delta} is the parameter, d_k(i) and d_k(j)
are the distances from node i or node j to their k nearest neighbor.
}
\value{
An n by n binary dgCMatrix object C, where n is the number of input samples.
The matrix C is the adjacency matrix of the built CKNN graph. C[i,j] = 1
means that there is an edge between sample i and sample j.
}
\examples{
data(Pollen)
Pollen.PCs <- Pollen[["PCs"]]
G <- CKNN.Construction(Pollen.PCs)
}

\name{HGC.PlotARIs}
\alias{HGC.PlotARIs}
\title{
Calculating and Visualizing ARIs of the clustering results with given labels
}
\description{
The function cut the dendrogram into specific clusters at different levels
and compared the clusterings with given labels using Adjusted Rand Index (ARI)
}
\usage{
HGC.PlotARIs(tree, k.min, k.max, labels, return.ARI)
}
\arguments{
    \item{tree}{
    the input clustering tree saved as \code{hclust} data structure.
    }
    \item{k.min}{
    the minimum number to cut the tree.
    }
    \item{k.max}{
    the maximum number to cut the tree.
    }
    \item{labels}{
    a data frame or a matrix to store the label information. Different
    labels should be in different columns and the users should name the
    columns correspondingly.
    }
    \item{return.ARI}{
    a bool variable to choose whether output the ARI matrix.
    }
}
\details{
ARI is a widely used index to evaluate the consistence between two partitions
of the same samples. This function will first cut a given tree into specific
number of clusters using the function \code{cutree}. Then it calculates the
ARIs between the clustering result and the given labels
with the help of R package \code{mclust}. The function does such cutting and
calculation for different ks between k.min and k.max. Finally it visualize
these results using a line chart. ARIs with different labels are shown as
different lines with different colors in the figure.
}
\value{
A line chart will be drawn and a matrix of the ARIs will be returned.
}
\examples{
data(Pollen)
Pollen.PCs <- Pollen[["PCs"]]
Pollen.Label.Tissue <- Pollen[["Tissue"]]
Pollen.Label.CellLine <- Pollen[["CellLine"]]

Pollen.SNN <- SNN.Construction(Pollen.PCs)
Pollen.ClusteringTree <- HGC.dendrogram(G = Pollen.SNN)
Pollen.labels <- data.frame(Tissue = Pollen.Label.Tissue,
                            CellLine = Pollen.Label.CellLine)
HGC.PlotARIs(tree = Pollen.ClusteringTree,
                            k.min = 2, k.max = 15,
                            labels = Pollen.labels)
}

\name{HGC.PlotParameter}
\alias{HGC.PlotParameter}
\title{
Visualizing the Parameter Records during Clustering
}
\description{
The function visualizes the parameter output from \code{HGC.parameter}.
}
\usage{
HGC.PlotParameter(record, parameter)
}
\arguments{
    \item{record}{
    the input record matrix of parameters from \code{HGC.parameter}.
    }
    \item{parameter}{
    a string with alternatives "CL" or "ANN". Choose "CL" to plot the
    chain lengths and "ANN" to plot the average neighbor number.
    }
}
\details{
The chain length(CL) and average neighbor number(ANN) are key factors
related
with the time complexity of clustering by \code{HGC}. The function
provides the
visualization of them.
}
\value{
The function will return 1 if the dendrogram is successfully drawn.
}
\examples{
data(Pollen)
Pollen.PCs <- Pollen[["PCs"]]
Pollen.SNN <- SNN.Construction(Pollen.PCs)
Pollen.ParameterRecord <- HGC.parameter(G = Pollen.SNN)
HGC.PlotParameter(Pollen.ParameterRecord, parameter = "CL")
HGC.PlotParameter(Pollen.ParameterRecord, parameter = "ANN")
}

\name{PMST.Construction}
\alias{PMST.Construction}
\title{
Building Unweighted Perturbed Minimum Spanning Tree Graph
}
\description{
This function builds an Unweighted Perturbed Minimum Spanning Tree
(PMST) graph in the input feature space using Euclidean distance metric.
}
\usage{
PMST.Construction(mat, iter, r)
}
\arguments{
    \item{mat}{
    the input data saved as a numerical matrix. The columns are the
    features and the rows are the samples.
    }
    \item{iter}{
    the number of perturbation.
    }
    \item{r}{
    the parameter about the strength of the perturbation.
    }
}
\details{
The function builds a PMST graph of the input data. PMST is the
combination of a number of MSTs, which are built in the perturbed data
spaces.
}
\value{
An n by n binary dgCMatrix object C, where n is the number of input
samples.
The matrix C is the adjacency matrix of the built PMST graph. C[i,j] = 1
means that there is an edge between sample i and sample j.
}
\examples{
data(Pollen)
Pollen.PCs <- Pollen[["PCs"]]
G <- PMST.Construction(Pollen.PCs)
}

\name{SNN.Construction}
\alias{SNN.Construction}
\title{
Building Unweighted Shared Nearest Neighbor Graph
}
\description{
This function builds a Shared Nearest Neighbor (SNN) graph in the input 
feature space using Euclidean distance metric.
}
\usage{
SNN.Construction(mat, k, threshold)
}
\arguments{
    \item{mat}{
    the input data saved as a numerical matrix. The columns are the
    features and the rows are the samples.
    }
    \item{k}{
    the number of nearest neighbor number to build the original KNN.
    }
    \item{threshold}{
    the threshold parameter for the Jaccard index. The edges in KNN
    whose Jaccard indices are lower than it will be removed in building 
    the SNN.
    }
}
\details{
The function builds an SNN which saved as a sparse matrix.
}
\value{
An n by n binary dgCMatrix object C, where n is the number of input 
samples.
The matrix C is the adjacency matrix of the built SNN graph. C[i,j] = 1  
means that there is an edge between sample i and sample j.
}
\examples{
data(Pollen)
Pollen.PCs <- Pollen[["PCs"]]
G <- SNN.Construction(Pollen.PCs)
}
